import subprocess
from typing import List


class ProcessRunnerPort:
    def run_process(self, process: List[str]) -> None:
        pass


class ProcessRunnerAdapter(ProcessRunnerPort):
    def run_process(self, process: List[str]) -> None:
        subprocess.run(process)



from symeo_python.cli.cli import DefaultCliAdapter
from symeo_python.cli.process_runner import ProcessRunnerAdapter
from symeo_python.cli.symeo_python_cli import SymeoPythonCli
from symeo_python.configuration.config_parser import ConfigParserAdapter
from symeo_python.yaml_converter.yaml_to_class_converter import YamlToClassAdapter

cli = SymeoPythonCli(
    DefaultCliAdapter(ConfigParserAdapter(YamlToClassAdapter()), ProcessRunnerAdapter())
).load_commands()

if __name__ == "__main__":
    cli()



import os

from symeo_python.cli.process_runner import ProcessRunnerPort
from symeo_python.configuration.config_loader import (
    SYMEO_API_KEY,
    SYMEO_LOCAL_FILE,
    SYMEO_API_URL,
)
from symeo_python.configuration.config_parser import ConfigParserPort


class CliPort:
    def generate_configuration_from_contract_file(self, config_contract: str):
        pass

    def prepare_env_and_start_sub_process(self, cli_input_data: dict):
        pass


class DefaultCliAdapter(CliPort):
    def __init__(
        self, conf_parser_port: ConfigParserPort, process_runner_port: ProcessRunnerPort
    ):
        self.__conf_parser_port = conf_parser_port
        self.__process_runner_port = process_runner_port

    def generate_configuration_from_contract_file(self, config_contract: str):
        self.__conf_parser_port.generate_configuration(config_contract)

    def prepare_env_and_start_sub_process(self, cli_input_data: dict):
        if "api_key" in cli_input_data and "api_url" in cli_input_data:
            os.environ[SYMEO_API_KEY] = cli_input_data["api_key"]
            os.environ[SYMEO_API_URL] = cli_input_data["api_url"]
        elif "config_values_path" in cli_input_data:
            os.environ[SYMEO_LOCAL_FILE] = cli_input_data["config_values_path"]
        else:
            raise Exception("Missing api_key/api_url or config_values_path")
        self.__process_runner_port.run_process(cli_input_data["sub_process"])



import os
from typing import List

import yaml  # type: ignore


class YamlToClassPort:
    def parse_configuration_from_path(self, configuration_path):
        pass


class YamlToClassAdapter(YamlToClassPort):

    __target_file_path: str
    __ROOT_CONFIG_CLASS_NAME = "Config"
    __TAB = "    "

    def __init__(
        self,
        target_file_path: str = f"{os.path.dirname(os.path.abspath(__file__))}/../configuration/configuration.py",
    ):
        self.__target_file_path = target_file_path

    def parse_configuration_from_path(self, configuration_path):
        with open(configuration_path, "r") as yaml_file:
            yaml_data = yaml.load(yaml_file, Loader=yaml.FullLoader)
            class_definition = self.__yaml_data_to_class_with_mapping_methods(
                yaml_data, self.__ROOT_CONFIG_CLASS_NAME, []
            )
            with open(self.__target_file_path, "w") as file:
                print(f"Writing conf from contract to {configuration_path}")
                file.write(class_definition)

    def __yaml_data_to_class_with_mapping_methods(
        self, yaml_data: dict, class_name: str, nested_classes: List[str]
    ) -> str:
        attribute = ""
        constructor = f"{self.__TAB}def __init__(self, yaml_data):\n"
        (
            attribute,
            constructor,
        ) = self.__yaml_data_to_attribute_constructor_and_nested_classes(
            attribute, constructor, nested_classes, yaml_data
        )
        conf_class = ""
        if class_name == self.__ROOT_CONFIG_CLASS_NAME:
            python_conf_file_content = ""
            for nested_classe in nested_classes:
                python_conf_file_content += f"{nested_classe}\n\n"
            conf_class = python_conf_file_content
        conf_class += f"class {class_name}:\n{attribute}\n{constructor}"
        return conf_class

    def __yaml_data_to_attribute_constructor_and_nested_classes(
        self, attribute, constructor, nested_classes, yaml_data
    ):
        for key in yaml_data:
            attribute_name = str(key).replace("-", "_")
            if "type" in yaml_data[key]:
                constructor += f'{self.__TAB}{self.__TAB}self.{attribute_name} = yaml_data["{key}"]\n'
                type = self.__get_type(yaml_data[key]["type"])
                attribute += f"{self.__TAB}{attribute_name}: {type}\n"
            else:
                nested_class_name = self.__get_camel_cased_class_name(key)
                constructor += f'{self.__TAB}{self.__TAB}self.{attribute_name} = {nested_class_name}(yaml_data["{key}"])\n'
                nested_classes.append(
                    self.__yaml_data_to_class_with_mapping_methods(
                        yaml_data[key], nested_class_name, nested_classes
                    )
                )
                attribute += f"{self.__TAB}{attribute_name}: {nested_class_name}\n"
        return attribute, constructor

    @staticmethod
    def __get_camel_cased_class_name(class_name):
        camel_case_class_name = ""
        if "-" in class_name:
            for sub_name in class_name.split("-"):
                camel_case_class_name += sub_name.capitalize()
        else:
            camel_case_class_name = class_name.capitalize()
        return camel_case_class_name

    @staticmethod
    def __get_type(type: str) -> str:
        if type == "string":
            return "str"
        elif type == "integer":
            return "int"
        raise Exception(f"Wrong type {type}")



import os
import shutil
import tempfile
import unittest

import yaml

from test.yaml_converter.resources.expected_python_files.complex_yaml_with_complex_class_name import (
    Config,
)

# import yaml  # type: ignore

from symeo_python.configuration.config_parser import ConfigParserAdapter
from symeo_python.yaml_converter.yaml_to_class_converter import YamlToClassAdapter


class YamlToClassConverterTest(unittest.TestCase):

    __temp_dir: str
    __current_absolute_path = os.path.dirname(os.path.abspath(__file__))

    def setUp(self) -> None:
        self.__temp_dir = tempfile.mkdtemp()

    def tearDown(self) -> None:
        shutil.rmtree(self.__temp_dir)

    def test_should_convert_simple_yaml_files(self):
        self.__assert_generated_python_file_equals_expected_python_file(
            "simple_yaml_1.yml", "simple_yaml_1.py"
        )
        self.__assert_generated_python_file_equals_expected_python_file(
            "simple_yaml_2.yml", "simple_yaml_2.py"
        )
        self.__assert_generated_python_file_equals_expected_python_file(
            "simple_yaml_with_two_attributes.yml", "simple_yaml_with_two_attributes.py"
        )

    def test_should_convert_complex_yaml_files(self):
        self.__assert_generated_python_file_equals_expected_python_file(
            "yaml_with_nested_conf.yml", "yaml_with_nested_conf.py"
        )

    def test_should_convert_complex_name(self):
        self.__assert_generated_python_file_equals_expected_python_file(
            "yaml_with_complex_name.yml", "yaml_with_complex_name.py"
        )
        self.__assert_generated_python_file_equals_expected_python_file(
            "complex_yaml_with_complex_class_name.yml",
            "complex_yaml_with_complex_class_name.py",
        )

    def test_should_load_complex_yaml_to_class(self):
        # Given
        yaml_file_name = "complex_yaml_with_complex_class_name.yml"
        given_conf_file = f"{self.__current_absolute_path}/resources/given_conf_yaml_files/{yaml_file_name}"

        with open(given_conf_file, "r") as yaml_file:
            yaml_data = yaml.load(yaml_file, Loader=yaml.FullLoader)
            config = Config(yaml_data)
            self.assertEqual(config.application_name, "symeo-python")
            self.assertEqual(
                config.infrastructure.database.mongo_db.url, "http://localhost"
            )
            self.assertEqual(config.infrastructure.database.mongo_db.port, 5132)
            self.assertEqual(config.infrastructure.name, "fake")
            self.assertEqual(config.infrastructure.github.api, "http://github.api.com")

    def __assert_generated_python_file_equals_expected_python_file(
        self, yaml_file_name, python_file_name
    ):
        given_yaml_file = f"{self.__current_absolute_path}/resources/given_contract_yaml_files/{yaml_file_name}"
        target_path = f"{self.__temp_dir}/%s" % python_file_name
        yaml_to_class_converter = YamlToClassAdapter(target_path)
        expected_python_file = f"{self.__current_absolute_path}/resources/expected_python_files/{python_file_name}"
        # When
        yaml_to_class_converter.parse_configuration_from_path(given_yaml_file)

        # Then
        self.assertEqual(
            ConfigParserAdapter.get_file_md5(expected_python_file),
            ConfigParserAdapter.get_file_md5(target_path),
        )
